
const int ID_USER=20000;

const int IDM_RPT_PRINT		= ID_USER+200;
const int IDM_RPT_OPTION	= ID_USER+201;	// ...through 220
const int IDM_RPT_CLOSE		= ID_USER+221;

const int IDM_RPT_ZOOMIN	= ID_USER+251;
const int IDM_RPT_ZOOMNONE	= ID_USER+252;
const int IDM_RPT_ZOOMOUT	= ID_USER+253;

const int IDM_RPT_PAGEFIRST	= ID_USER+271;
const int IDM_RPT_PAGEUP	= ID_USER+272;
const int IDM_RPT_PAGEDOWN	= ID_USER+273;	
const int IDM_RPT_PAGELAST	= ID_USER+274;

const int IDM_RPT_ABOUT		= ID_USER+281;

//-----------------------------------------------------------------------------
enum RPTOBJTYPE { RPT_LINE, RPT_BOX, RPT_TEXT, RPT_RGRID };

//-----------------------------------------------------------------------------
class CRptObject : public CItem { 
	public : 
		int m_iX0, m_iY0, m_iX1, m_iY1;
		COLORREF m_rgb;
		RPTOBJTYPE m_iType;

		CRptObject( int iX0, int iY0, int iX1, int iY1, COLORREF rgb );
		virtual void WritePdf( FILE * hOut );		// Write to Rpt file 
		virtual void Draw( HDC hdc );
		virtual int PDFSize( );
		void WritePdf_rg( FILE * hOut, COLORREF rgb, bool bStroke=true );
		int Size_rg( );			
	};


//-----------------------------------------------------------------------------
// Todo - colors, stroke Width
class CRptLine : public CRptObject { 
	public : 
		int m_iWidth;		// in 1/72 inches
		CRptLine( int iX0, int iY0, int iX1, int iY1, COLORREF rgb, int iWidth=-1 );
		virtual int PDFSize( );
		virtual void WritePdf( FILE * hOut );
		virtual void Draw( HDC hdc );
	};

//-----------------------------------------------------------------------------
class CRptBox : public CRptObject { 
	public : 
		COLORREF m_rgbFill;
		int m_iWidth;		// in 1/72 inches
		bool m_bFill, m_bStroke;

		CRptBox( int iX0, int iY0, int iX1, int iY1, bool bStroke, bool bFill, COLORREF rgbEdge, COLORREF rgbFill=RGB(255,255,255), int iWidth=-1 );
		virtual void WritePdf( FILE * hOut );
		virtual void Draw( HDC hdc );
		virtual int PDFSize( );
	};

//-----------------------------------------------------------------------------
class CRptText : public CRptObject { 
	public : 
		int m_iSize;
		char * m_pchText;
		int m_iAngle;
		bool m_bBold;
		bool m_bFixed;

		CRptText( int iX0, int iY0, int iSize, const char * pch, COLORREF rgb );
		~CRptText();
		virtual void WritePdf( FILE * hOut );
		virtual int PDFSize( );	
		virtual void Draw( HDC hdc );
	};

//-----------------------------------------------------------------------------
class CRptGroup : public CRptObject { 
	public :
		CItemArray m_aObjects;

		CRptGroup( int iX0, int iY0, int iX1, int iY1, COLORREF rgb );
		CRptGroup( );
		virtual void WritePdf( FILE * hOut );			
		virtual void Draw( HDC hdc );
		virtual int PDFSize( );	
		
		void AddLine( int iX0, int iY0, int iX1, int iY1, COLORREF rgb=RGB(0,0,0), int iWidth=-1 );
		void AddText( int iSize, int iX0, int iY0, const char * pch, COLORREF rgb);
		void AddBox( int iX0, int iY0, int iX1, int iY1, bool bStroke, bool bFill, COLORREF rgbEdge=RGB(0,0,0), COLORREF rgbFill=RGB(255,255,255) ) ;
		
	};
	
//-----------------------------------------------------------------------------
const int HEADSEP = 2;			// separation between headers and column headers
const int HEADOFFSET = 2;		// offset from edge to left of headings
const int COLSEP = 4;			// separation between adjacent columns
const int COLOFFSET = 2;		// offset from edge to left of 1st column

class CRptRGridCol : public CItem { 
	public : 
		int m_iPos;
		int m_iWidth;

		CRptRGridCol( int iPos, int iWidth );
	};

class CRptRGrid : public CRptGroup { 
	public :
		int m_iHeadRows;
		int m_iHeadRowHeight;			// default to 10, change after construction if desired
		int m_iHeadRowFontSize;
		int m_iColHeadRows;
		int m_iColHeadRowsHeight;		// default to 10, change after construction if desired
		int m_iColHeadFontSize;
		int m_iDataRowHeight;			// default to 10, change after construction if desired
		int m_iDataFontSize;
		COLORREF m_rgbDefault;			// default to black, change after construction if desired

		//--
		CRptRGrid( int iX0, int iY0, int iY1, int iHeadRows, int iColHeadRows );	

		int DataOffset( int iRow=0 );
		
		//-- Overall setup 
		CRptText * AddHeadText( int iHeadRow, const char * pchText, COLORREF rgb=RGB(0,0,0), int iOffset=0 );
		void AddBorder();

		//-- Columns setup 
		int m_iWidth;			// width of columns so far = offset to next column
		CItemArray m_aColumns; 
		void AddCol( int iWidth );
		CRptText * AddColHeadText( int iCol, int iHeadRow, const char * pchText, COLORREF rgb=RGB(0,0,0) );

		//-- Column data contents
		//TODO - Right justify data 
		CRptText * AddDataItem( int iCol, int iRow, const char * pchText, COLORREF rgb=RGB(0,0,0) );

		//-- Add horizontal separator above data row iRow for column iCol or all columns if iCol=-1;
		void AddHSep( int iRow, int iCol=-1 );
	};

//-----------------------------------------------------------------------------
class CRptPage : public CItem { 
	public : 
		CRptGroup m_oGroup;
		//CItemArray m_aObjects;		// array of drawn objects, NULL is possible
		CRptObject * Object( int iObject );

		int m_iRefPos, m_iContentPos;
		void WritePdf( FILE * hOut ) ;
		int PDFSize( ) ;
		void AddLine( int iX0, int iY0, int iX1, int iY1, COLORREF rgb=RGB(0,0,0), int iWidth=-1 );
		void AddText( int iSize, int iX0, int iY0, const char * pch, COLORREF rgb);
		void AddBox( int iX0, int iY0, int iX1, int iY1, bool bStroke, bool bFill, COLORREF rgbEdge=RGB(0,0,0), COLORREF rgbFill=RGB(255,255,255) ) ;
		void Draw( HDC hdc );
	};

//-----------------------------------------------------------------------------
class CRptWin { 
	private :
		CItemArray m_aPages;
		int m_iPage, m_iZoom;
		int m_iXSize, m_iYSize, m_iXOrg, m_iYOrg;
	public :
		CRptPage * LastPage( ) ;
		CRptPage * AddPage( );
		CRptPage * Page( int iPage );
		bool WritePdf( const char * pchFile ) ;
		
		bool DrawPage( int iPage, HDC hdc );

		HWND m_hWnd;		
		static HINSTANCE m_hInst;
		static BOOL WinInit( HINSTANCE hParent, BOOL lBlack );
		static long WINAPI WinProc( HWND hWnd, UINT msg, WPARAM wP, LPARAM lP );
		BOOL WinOpen( const char * pchTitle, bool bPrint, CStringArray * pacOptions );
		
	};

void TestPDFRpt();




/*
void ChartFrame( int * aiX, int * aiY, 
		double dXMin, double dXMax, 
		double dYMin, double dYmax, 
		int iXLbls, double * adXLblPos, char ** apchXLblText, 
		int iYLbls, double * adYLblPos, char ** apchYLblText, 
		char * pchTitle, 
		char * pchXTitle, 
		char * pchYTitle ) { 
	int i, iX, iY;
	pPage->AddBox( aiX[4], aiY[5], aiX[5], aiY[6], true, false, RGB(0,0,0), RGB(128,128,128) );

	pPage->AddText( 14, aiX[4], aiY[7], pchTitle, RGB(0,0,0) );
	pPage->AddText( 10, aiX[4], aiY[2], pchXTitle, RGB(0,0,0) );
	pPage->AddText( 10, aiX[0], aiY[5], pchYTitle, RGB(0,0,0) );
	for( i=0; i<iXLbls; ++i ) { 
		iX = (adXLblPos[i]-dXMin)/(dXMax-dXMin)*double(aiX[5]-aiX[4]);
		pPage->AddLine( iX, aiY[5], iX, aiY[4] );
		pPage->AddText( 10, iX, aiY[2], apchXLblText[i], RGB(0,0,0) );
		}
	for( i=0; i<iYLbls; ++i ) { 
		iY = (adYLblPos[i]-dYMin)/(dYMax-dYMin)*double(aiY[6]-aiY[5]);
		pPage->AddLine( aiX[3], iY, aiX[4], iY );
		pPage->AddText( 10, aiX[2], iY, apchYLblText[i], RGB(0,0,0) );
		}
		
	pPage->AddLine( aiX[4], aiY[5], aiX[5], aiY[6], RGB(0,0,0) );
	}
*/
